import React, {
  useCallback,
  useState,
  useMemo,
} from 'react';
import {
  scaleTime,
  extent,
  scaleLog,
  max,
  line,
  timeFormat,
  format,
} from 'd3';
import { XAxis } from './XAxis';
import { YAxis } from './YAxis';
import { VoronoiOverlay } from './VoronoiOverlay';
import { Tooltip } from './Tooltip';

const xValue = (d) => d.date;
const yValue = (d) => d.deathTotal;

const margin = { top: 30, right: 60, bottom: 70, left: 70 };

export const LineChart = ({ data, width, height }) => {
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  const [activeRow, setActiveRow] = useState();

  const allData = useMemo(
    () => data.reduce((acc, val) => acc.concat(val), []),
    [data]
  );

  const xScale = useMemo(
    () =>
      scaleTime()
        .domain(extent(allData, xValue))
        .range([0, innerWidth]),
    [allData, xValue, innerWidth]
  );

  const yScale = useMemo(
    () =>
      scaleLog()
        .domain([1, max(allData, yValue)])
        .range([innerHeight, 0]),
    [allData, yValue, innerHeight]
  );

  const lineGenerator = useMemo(
    () =>
      line()
        .x((d) => xScale(xValue(d)))
        .y((d) => yScale(1 + yValue(d))),
    [xScale, xValue, yScale, yValue]
  );

  const mostRecentDate = xScale.domain()[1];
  const formatDate = timeFormat('%b %d, %Y');

  const handleVoronoiHover = useCallback(setActiveRow, []);

  console.log(activeRow);

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left},${margin.top})`}
      >
        <XAxis xScale={xScale} innerHeight={innerHeight} />
        <YAxis yScale={yScale} innerWidth={innerWidth} />
        {data.map((country) => (
          <path
            className="marker-line"
            d={lineGenerator(country)}
          />
        ))}
        {activeRow ? (
          <>
            <path
              className="marker-line active"
              d={lineGenerator(
                data.find(
                  (countryTimeSeries) =>
                    countryTimeSeries.countryName ===
                    activeRow.countryName
                )
              )}
            />
            <g
              transform={`translate(${lineGenerator.x()(
                activeRow
              )},${lineGenerator.y()(activeRow)})`}
            >
              <circle r={5} />
              <Tooltip activeRow={activeRow} className='tooltip tooltip-stroke'/>
              <Tooltip activeRow={activeRow} className='tooltip'/>
              
            </g>
          </>
        ) : null}
        <text className='title-text'
          
        >
          Global Coronavirus Deaths by Country
        </text>
        <text
          className="axis-label"
          transform={`translate(-40, ${
            innerHeight / 2
          }) rotate(-90)`}
          text-anchor="middle"
        >
          Cumulative Deaths
        </text>
        <text
          className="axis-label"
          transform={`translate(${innerWidth / 2},${
            innerHeight + 40
          })`}
          alignment-baseline="hanging"
          text-anchor="middle"
        >
          Time
        </text>

        <VoronoiOverlay
          onHover={handleVoronoiHover}
          innerWidth={innerWidth}
          innerHeight={innerHeight}
          allData={allData}
          lineGenerator={lineGenerator}
          margin={margin}
        />
      </g>
    </svg>
  );
};
